# -*- coding: utf-8 -*-
"""
Created on Sat Feb 16 08:51:31 2019

@author: michaelek
"""
import numpy as np
import pandas as pd
from allotools.filters import allo_filter

#####################################
### Functions








